/* filelist.h */

#ifndef __FILELIST_H
#define __FILELIST_H

#include <glib.h>

#include "player.h"

typedef struct
{
   gchar *filename;
   gboolean isdir : 1;
   gboolean islist : 1;
   player_type player : 6;
} filelist_element;

typedef struct
{
   GList *list;
   gchar *dirname;
   gint olddir;
   
   gchar *pattern[PLAYER_LAST];
   GList *lastacc; /* pointer to last accessed element in list (or NULL if invalid )*/
   gint nlastacc; /* index of last accessed element in list (or -1 if invalid)*/
} FileListData;

typedef gint (*PlaylistAddFunc)( gpointer data, const gchar *dirname, 
                                 const gchar *filename, player_type player );

/* private */
char* reldir2abs( const char *reldir );
void filelist_print_e( filelist_element *fle, gpointer user_data );
void filelist_free_e( filelist_element *fle );
gint filelist_compare( filelist_element *fle1, filelist_element *fle2 );
gint compare_random( gpointer *p1, gpointer *p2 );

void filelist_register_pattern( FileListData *data, player_type player, const gchar *pattern );

int filelist_add_files( FileListData *data, player_type player, const char *pattern );

int filelist_add_dirs( FileListData *data );

void filelist_free( FileListData *data );

void filelist_print( FileListData *data );

void filelist_add_all( FileListData *data );

gboolean filelist_nth_isfile( FileListData *data, guint n );
gboolean filelist_nth_isdir( FileListData *data, guint n );
gboolean filelist_nth_islist( FileListData *data, guint n );

char* filelist_get_nth_name( FileListData *data, guint n );

char* filelist_get_nth_absname( FileListData *data, guint n );

player_type filelist_get_nth_type( FileListData *data, guint n );

guint filelist_get_length( FileListData *data );

gint filelist_get_index( FileListData *data, gchar *file );

char* filelist_get_dirname( FileListData *data );

gint filelist_get_olddir( FileListData *data );

gint filelist_open_dir( FileListData *data, const char* dirname );

gint filelist_add_dir_r( FileListData *data, const gchar *dirname, PlaylistAddFunc addfunc, gpointer funcdata );
gint filelist_open_list( FileListData *data, const gchar *filename, PlaylistAddFunc addfunc, gpointer funcdata );
gint filelist_open_unknown( FileListData *data, const gchar *filename, PlaylistAddFunc addfunc, gpointer funcdata );

FileListData* filelist_new();

#endif /* __FILELIST_H */
