/* logger.c */

#include <stdarg.h>
#include <errno.h>

#include "config.h"
#include "logger.h"

#define LOGGER_CLOSE (1<<8)

void g_debug( const gchar *format, ... )
{
   va_list args;
   
   va_start(args, format);
   g_logv(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, format, args);
   va_end(args);
}

void g_info( const gchar *format, ... )
{
   va_list args;
   
   va_start(args, format);
   g_logv(G_LOG_DOMAIN, G_LOG_LEVEL_INFO, format, args);
   va_end(args);
}

void logger_open( const gchar *logfile )
{
   LoggerData *data;
   
   data = g_new(LoggerData, 1);
   data->logfile = fopen(logfile, "a");
   if( data->logfile == NULL )
   {
      g_error("Cannot open logfile '%s': %s", logfile, g_strerror(errno));
      return;
   }
   
   g_log_set_handler(NULL,
                     /* G_LOG_LEVEL_ERROR| */
                     G_LOG_LEVEL_CRITICAL|
                     G_LOG_LEVEL_WARNING|
                     G_LOG_LEVEL_MESSAGE|
                     G_LOG_LEVEL_INFO|
                     G_LOG_LEVEL_DEBUG|
                     LOGGER_CLOSE,
                     (GLogFunc)logger_log, data);
}

void logger_close( void )
{
   g_log(G_LOG_DOMAIN, LOGGER_CLOSE, " ");
}

void logger_log( const gchar *domain, GLogLevelFlags log_level,
                 const gchar *message, LoggerData *data )
{
   gchar *level[] = { "** ERROR **: ", "** CRITICAL **: ", "** WARNING **: ", "Message: ", "Info: ", "DEBUG: ", "" };
   gint l=6;
   
   if( data->logfile == NULL )
   {
      fprintf(stderr, "Error in logger_log(): logfile not open.\n");
      return;
   }
   
   if( log_level & LOGGER_CLOSE )
   {
      fclose(data->logfile);
      data->logfile = NULL;
      g_free(data);
      return;
   }
   
   if( log_level & G_LOG_LEVEL_ERROR )
      l = 0;
   else if( log_level & G_LOG_LEVEL_CRITICAL )
      l = 1;
   else if( log_level & G_LOG_LEVEL_WARNING )
      l = 2;
   else if( log_level & G_LOG_LEVEL_MESSAGE )
      l = 3;
   else if( log_level & G_LOG_LEVEL_INFO )
      l = 4;
   else if( log_level & G_LOG_LEVEL_DEBUG )
      l = 5;
   
   fprintf(data->logfile, "%s%s%s%s%s\n",
           (l<3)?"\n":"", (domain)?domain:"", (domain)?"-":"", level[l], message);
   fflush(data->logfile);
}
