/* mixer.c */

#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <linux/soundcard.h>
#include <errno.h>
#include <string.h>

#include "config.h"
#include "logger.h"
#include "mixer.h"


MixerData* mixer_new()
{
   MixerData *data;
   gint mixfd;
   
   data = g_new0(MixerData,1);
   
   if( (mixfd=open(MIXER_DEV, O_RDWR)) < 0 )
   {  g_warning(__FILE__": Error opening mixer device '"MIXER_DEV"': %s", g_strerror(errno));
      return NULL;
   }
   
   data->mixfd = mixfd;
   
   return data;
}

void mixer_free( MixerData *data )
{
   if( close(data->mixfd) < 0 )
   {  g_warning(__FILE__": Error closing mixer device '"MIXER_DEV"': %s", g_strerror(errno));
   }
   
   g_free(data);
}

void mixer_set_volume( MixerData *data, gint vol )
{
  guchar temp[4];
  
  g_return_if_fail(data != NULL);
  g_return_if_fail(data->mixfd > 0);
  
  vol = CLAMP(vol, 0, 100);

  temp[0]=(guchar)vol; /* left */
  temp[1]=(guchar)vol; /* right */
  temp[2]=temp[3]=0;
  
  ioctl(data->mixfd, SOUND_MIXER_WRITE_VOLUME, temp);
}

gint mixer_get_volume( MixerData *data )
{
  guchar temp[4];

  g_return_val_if_fail(data != NULL, -1);
  g_return_val_if_fail(data->mixfd > 0, -1);

  ioctl(data->mixfd, SOUND_MIXER_READ_VOLUME, &temp);
  
  /* since we return _one_ value let
     us return the medium of left & right */
  return (temp[0]+temp[1])/2;
}

gint mixer_inc_volume( MixerData *data, gint step )
{
   gint vol;
   
   vol = mixer_get_volume(data);
   vol += step;
   vol = CLAMP(vol, 0, 100);
   mixer_set_volume(data, vol);
   
   return vol;
}

/*--------------------------------------------------------*/

#ifdef SAL
int main(int argc, char *argv[])
{
   int v;
   
   if((mixfd=open(MIXER_DEV, O_RDWR)) < 0)
   {  perror("Fehler in open device");
      return -1;
   }
   
   if(argc == 1)
   {  get_volume(&v);
      printf("Got Volume=%d\n", v);
   }
   else if(argc == 2)
   {  v = atoi(argv[1]);
      set_volume(v);
      printf("Set Volume=%d\n", v);
   }
   else
   {  printf("Usage: %s [<vol>]\n", argv[0]);
      return -1;
   }
   
   close(mixfd);
   
   return 0;
}
#endif /* SAL */
