/* player.h */

/* interface to several player types */

#ifndef __PLAYER_H
#define __PLAYER_H

#include <glib.h>

typedef enum
{
   PLAYER_NONE = 0,
   PLAYER_MP3,
   PLAYER_OGG,
   PLAYER_SOX,
   PLAYER_MID,
   PLAYER_LAST   /* *must* be the last */
} player_type;

/* functions that should be provided by each player type */
typedef gint (*InitFunc)(gpointer data);
typedef gint (*PlayFunc)(gpointer data, const gchar *filename);
typedef gint (*StopFunc)(gpointer data);
typedef gint (*SuspendFunc)(gpointer data);
typedef gint (*ContinueFunc)(gpointer data);
typedef gint (*SeekFunc)(gpointer data, gdouble seconds);
typedef gint (*ProgressFunc)(gpointer data, gdouble *seconds_elapsed, gdouble *seconds_remaining);
typedef gint (*CloseFunc)(gpointer data);

/* function the player type should call if song is finished */
/* it returns FALSE so it can be used in g_timeout_add() */
typedef gboolean (*GimmeNextFunc)(gpointer user_data);

typedef void (*GetNextFunc)(gpointer user_data);

typedef struct _PlayerInterface
{
   InitFunc 	player_init;
   PlayFunc 	player_play;
   StopFunc 	player_stop;
   SuspendFunc 	player_suspend;
   ContinueFunc player_continue;
   SeekFunc 	player_seek;
   ProgressFunc player_progress;
   CloseFunc 	player_close;
   gpointer data;
   
   GimmeNextFunc player_gimme_next;
   gpointer user_data;
   
   gchar *pattern;
} PlayerInterface;

typedef struct _PlayerData
{
   PlayerInterface player[PLAYER_LAST];   
   player_type currtype;
   
   GetNextFunc get_next;
   gpointer next_data;
   
   gboolean ispaused;
} PlayerData;

PlayerData* player_new();
void player_free( PlayerData *data );

gint player_init( PlayerData *data, player_type type );
gint player_close( PlayerData *data );

gint player_play( PlayerData *data, player_type type, const gchar *filename );
gint player_stop( PlayerData *data );
gint player_toggle_pause( PlayerData *data );
gint player_seek( PlayerData *data, gdouble seconds );
gint player_get_progress( PlayerData *data, gdouble *seconds_elap, gdouble *seconds_rem );

gchar* player_get_pattern( PlayerData *data, player_type type );
void player_set_next_func( PlayerData *data, GetNextFunc nextfunc, gpointer user_data );
gboolean player_ispaused( PlayerData *data );

#endif /* __PLAYER_H */
