/* playlist.h */

#ifndef __PLAYLIST_H
#define __PLAYLIST_H

#include <glib.h>

#include "player.h"

typedef struct _playlist_element
{
   gchar *absfilename;
   gchar *descript;
   gboolean ismarked : 1;
   gboolean isplaying : 1;
   /* gboolean noinfo : 1; */
   player_type player : 6;
} playlist_element;

typedef struct _PlayListData
{
   GList *list;
   GList *rand; /* contains same elements in random order */
   
   /* private data (used for speed) */
   GList *lastacc; /* pointer to last accessed element in list (or NULL if invalid )*/
   gint nlastacc; /* index of last accessed element in list (or -1 if invalid)*/
} PlayListData;

void playlist_clear( PlayListData *data );

void playlist_free( PlayListData *data );

void playlist_sort( PlayListData *data, gint *pos );

void playlist_shuffle( PlayListData *data, gint *pos );

void playlist_randomize( PlayListData *data, gint n );

void playlist_add( PlayListData *data, const char *dirname,
                   const char *filename, player_type player );

void playlist_delete( PlayListData *data, gint *pos );

void playlist_move( PlayListData *data, guint *pos1, gint *pos2 );

gint playlist_write( PlayListData *data, gchar *filename );
                   
char* playlist_get_nth_name( PlayListData *data, guint n );

char* playlist_get_nth_absname( PlayListData *data, guint n );

player_type playlist_get_nth_type( PlayListData *data, guint n );

gboolean playlist_get_ismarked( PlayListData *data, guint n );

void playlist_set_ismarked( PlayListData *data, guint n, gboolean ismarked );

void playlist_set_ismarked_regex( PlayListData *data, gboolean ismarked, const char *regex );

guint playlist_get_length( PlayListData *data );

gint playlist_rand2i( PlayListData *data, guint *n );

gint playlist_i2rand( PlayListData *data, guint *n );

PlayListData* playlist_new();

/* private helper functions */
playlist_element* playlist_get_element( PlayListData *data, guint n );

#endif /* __PLAYLIST_H */
/* EOF */
